@@ -58,6 +58,7 @@ struct _WnckScreenPrivate
   guint need_update_workspace_list : 1;
   guint need_update_active_workspace : 1;
   guint need_update_active_window : 1;
+  guint need_update_workspace_names : 1;
 };
 
 enum {
@@ -81,6 +82,7 @@ static void update_client_list      (WnckScreen      *screen);
 static void update_workspace_list   (WnckScreen      *screen);
 static void update_active_workspace (WnckScreen      *screen);
 static void update_active_window    (WnckScreen      *screen);
+static void update_workspace_names  (WnckScreen      *screen);
 
 static void queue_update            (WnckScreen      *screen);
 static void unqueue_update          (WnckScreen      *screen);              
@@ -276,6 +278,7 @@ wnck_screen_construct (WnckScreen *screen,
   screen->priv->need_update_stack_list = TRUE;
   screen->priv->need_update_active_workspace = TRUE;
   screen->priv->need_update_active_window = TRUE;
+  screen->priv->need_update_workspace_names = TRUE;
   
   queue_update (screen);
 }
@@ -332,7 +335,7 @@ wnck_screen_get_for_root (gulong root_window_id)
   i = 0;
   while (i < ScreenCount (gdk_display))
     {
-      if (screens[i]->priv->xroot == root_window_id)
+      if (screens[i] != NULL && screens[i]->priv->xroot == root_window_id)
         return screens[i];
       
       ++i;
@@ -479,6 +482,12 @@ _wnck_screen_process_property_notify (WnckScreen *screen,
       screen->priv->need_update_workspace_list = TRUE;
       queue_update (screen);
     }
+  else if (xevent->xproperty.atom ==
+           _wnck_atom_get ("_NET_DESKTOP_NAMES"))
+    {
+      screen->priv->need_update_workspace_names = TRUE;
+      queue_update (screen);
+    }
 }
 
 static gboolean
@@ -694,6 +703,9 @@ update_client_list (WnckScreen *screen)
       
       ++i;
     }
+  
+  g_free (stack);
+  g_free (mapping);
       
   /* put list back in order */
   new_list = g_list_reverse (new_list);
@@ -713,13 +725,13 @@ update_client_list (WnckScreen *screen)
       g_assert (created_apps == NULL);
       g_assert (closed_apps == NULL);
       g_list_free (new_stack_list);
-      g_list_free (new_list);
+      g_list_free (new_list);      
       --reentrancy_guard;
       return;
     }
 
   g_list_free (screen->priv->mapped_windows);
-  g_list_free (screen->priv->stacked_windows);
+  g_list_free (screen->priv->stacked_windows);  
   screen->priv->mapped_windows = new_list;
   screen->priv->stacked_windows = new_stack_list;
 
@@ -996,6 +1008,41 @@ update_active_window (WnckScreen *screen)
   emit_active_window_changed (screen);
 }
 
+static void
+update_workspace_names (WnckScreen *screen)
+{
+  char **names;
+  int i;
+  GList *tmp;
+  GList *copy;
+  
+  if (!screen->priv->need_update_workspace_names)
+    return;
+
+  screen->priv->need_update_workspace_names = FALSE;
+
+  names = _wnck_get_utf8_list (screen->priv->xroot,
+                               _wnck_atom_get ("_NET_DESKTOP_NAMES"));
+
+  copy = g_list_copy (screen->priv->workspaces);
+
+  i = 0;
+  tmp = copy;
+  while (tmp != NULL)
+    {
+      if (names && names[i])
+        _wnck_workspace_update_name (tmp->data, names[i]);
+      else
+        _wnck_workspace_update_name (tmp->data, NULL);
+
+      ++i;
+      tmp = tmp->next;
+    }
+
+  g_strfreev (names);
+  g_list_free (copy);
+}
+
 static void
 do_update_now (WnckScreen *screen)
 {
@@ -1012,6 +1059,7 @@ do_update_now (WnckScreen *screen)
   /* Then note any smaller-scale changes */
   update_active_workspace (screen);
   update_active_window (screen);
+  update_workspace_names (screen);
 }
 
 static gboolean
@@ -1125,4 +1173,9 @@ emit_application_closed (WnckScreen      *screen,
                  0, app);
 }
 
-
+gboolean
+wnck_screen_net_wm_supports (WnckScreen *screen,
+                             const char *atom)
+{
+  return gdk_net_wm_supports (gdk_atom_intern (atom, FALSE));
+}
