@@ -62,6 +62,8 @@ GtkWidget *wnck_tasklist_new (WnckScreen *screen);
 
 void wnck_tasklist_set_grouping (WnckTasklist             *tasklist,
 				 WnckTasklistGroupingType  grouping);
+void wnck_tasklist_set_switch_workspace_on_unminimize (WnckTasklist  *tasklist,
+						       gboolean       switch_workspace_on_unminimize);
 void wnck_tasklist_set_grouping_limit (WnckTasklist *tasklist,
 				       gint          limit);
 void wnck_tasklist_set_include_all_workspaces (WnckTasklist *tasklist,
